;Modified SI test ROM from outerworldarcade.com original
;Allows manual selecting individual sounds
;philmurr 01-22
;written in Z80 using only 8080-compatible instructions
;

;0000-1FFF  r   ROM code
;2000-23FF  rw  RAM
;2400-3FFF  rw  bitmapped screen (224x256)

;I/O Map

;00         r
;01         r   control inputs
;                bit 7
;                bit 6    right
;                bit 5    left
;                bit 4    fire
;                bit 3
;                bit 2    1 player start
;                bit 1    2 player start
;                bit 0    coin slot
;02         r   control inputs
;                bit 7    0=display coin info
;                bit 6    right 2
;                bit 5    left 2
;                bit 4    fire 2
;                bit 3    bonus (1500,1000) / preset mode
;                bit 2    1=tilt
;                bit 0-1  initial lives (3,4,5,6) / (3,4)
;02          w  shift count (0-7)
;03         r   shifted value (low then high)
;03          w  sound
;                bit 5    amp enable 
;                bit 4    bonus base
;                bit 3    invader hit
;                bit 2    base hit
;                bit 1    base fire
;                bit 0    saucer
;04          w  value to shift
;05          w  sound
;                bit 5    flip video
;                bit 4    saucer hit
;                bit 3    invader movement 4
;                bit 2    invader movement 3
;                bit 1    invader movement 2
;                bit 0    invader movement 1
;06          w  watchdog timer clear


xor a
out (0x03),a			;sound off

ld c,0x03				;loop 3 times
mainloop1: ld b,0x55		;preset b with 01010101
ld hl,0x2000			;fill 2000-3FFF with b
filloop1: ld (hl),b
inc hl
ld a,h
cp 0x40
jp nz,filloop1

ld hl,0x2000			;check 2000-3FFF contains b
checkloop1: ld a,(hl)
xor b
jp nz,memerr			;if different, jump out
inc hl
ld a,h
cp 0x40
jp nz,checkloop1

ld b,0xaa				;preset b with 10101010
ld hl,0x2000			;fill 2000-3FFF with b
filloop2: ld (hl),b
inc hl
ld a,h
cp 0x40
jp nz,filloop2

ld hl,0x2000			;check 2000-3FFF contains b
checkloop2: ld a,(hl)
xor b
jp nz,memerr			;if different, jump out
inc hl
ld a,h
cp 0x40
jp nz,checkloop2

dec c					;decrease counter
jp nz,mainloop1			;jump back if not done 3 times


filloopst: ld hl,0x2000	;fill 2000-3FFF with b, b contains 0->FF(E)
ld b,0
filloop3: ld a,b
ld (hl),a
inc hl
inc b
ld a,b
cp 0xff
jp nz,over1
ld b,0x00
over1: ld a,h
cp 0x40
jp nz,filloop3

ld hl,0x2000			;check 2000-3FFF contains b
ld b,0x00
checkloop3: ld a,(hl)
xor b
jp nz,memerr
inc b
ld a,b
cp 0xff
jp nz,over2			;if different, jump out
ld b,0x00
over2: inc hl
ld a,h
cp 0x40
jp nz,checkloop3
					;Programme RAM is ok, can use stack

ld sp,0x2400			;Set stack
call cls				;clear RAM
ld de,strings			;start of strings
ld hl,0x2501			;screen position
call pstring			;print ">ALL RAMS OK"
ld hl,0x2701
call pstring			;print ">CHECK INPUTS
ld hl,0x3301
call pstring			;print ">CHECK SOUND"

ld hl,0x2a07
ld c,0x0f
ld de,ports
push de
call psloop			;print "PORT1 01234567"
pop de
ld hl,0x2e07
ld c,0x0f
call psloop			;print "PORT1 01234567"
ld a,0x1d				;character "2"
ld hl,0x2e0b			;screen position
call pchar				;print "2" over last PORT1

ld b,0x0a				;10 strings
ld hl,0x340e			;screen position
ld de,inputs			;string start
pportloop: ld c,0x06		;6 characters
push	bc				;save bc
call psloop			;print UFO.F/MISSL/LAU.H/INV.H/EXTRA/INV.1/INV.2/INV.3/INV.4/UFO.H
ld bc,0x00fa			;next screen position
add hl,bc
pop bc				;retrieve bc
dec b
jp nz,pportloop			;jump back if not all printed


xor a
ld (0x2000),a
ld (0x2001),a
ld a,0x20
out (0x03),a			;amp on

sdloop: out (0x06),a		;watchdog
ld c,0x29				;"A" (space invader character)
call scursor			;print cursor
call pinput			;print inputs

in a,(0x01)			;read inputs port 1
and 64				;right
jp z,notr
ld a,(0x2001)			;button pressed flag
or a
jp nz,sdloop			;if button not been released, don't process
ld c,0x28				;" "
call scursor			;delete cursor
ld a,(0x2000)			;get cursor position
inc a
cp 10
jp nz,notend
xor a					;reset cursor to top if reached end
notend: ld (0x2000),a		;save new cursor position
ld a,1
ld (0x2001),a
jp sdloop

notr: in a,(0x01)		;read inputs port 1
and 32				;left
jp z,notl
ld a,(0x2001)			;button pressed flag
or a
jp nz,sdloop			;if button not been released, don't process
ld c,0x28				;" "
call scursor			;delete cursor
ld a,(0x2000)			;get cursor position
dec a
cp 255
jp nz,nottop
ld a,9				;set cursor end if reached top
nottop: ld (0x2000),a		;save new cursor position
ld a,1
ld (0x2001),a			;set button pressed flag
jp sdloop

notl: in a,(0x01)		;read inputs port 1
and 16				;fire
jp z,notf
ld a,(0x2001)			;button pressed flag
or a
jp nz,sdloop			;if button not been released, don't process
ld a,(0x2000)			;get cursor position
cp 5
jp nc,secondsnd			;jump if position >=5 to use second sound port
ld hl,sdata			;lookup table for sound to use (not elegant but it works!)
ld e,a
ld d,0
add hl,de
ld a,(hl)				;get sound to play
or 32					;ensure amp stays on
out (3),a
call pinputs
ld a,32
out (3),a				;turn sound off, amp stays on
jp sdloop

secondsnd:				;sounds for port 5
ld hl,sdata			;lookup table for sound
ld e,a
ld d,0
add hl,de
ld a,(hl)				;get sound to play
out (5),a				;output to port (amp is on already from port 3)
call pinputs
xor a
out (5),a				;and turn sound off
jp sdloop

notf: xor a
ld (0x2001),a			;reset button pressed flag
jp sdloop

sdata:
db 1,2,4,8,16,1,2,4,8,16,32


scursor:
ld a,(0x2000)
ld hl,0x340d
ld d,a
ld e,0
add hl,de				;calculate cursor position based on row
ld a,c				;character "A" or " "
jp pchar				;print on screen

pinputs: ld a,0x60
pinputloop: call pinput	;print inputs and watchdog
out (0x06),a			;watchdog
dec a
jp nz,pinputloop
ret     

pinput: push af			;save a
ld hl,0x2b0e			;screen position
in a,(0x01)			;read inputs port 1
call readinp			;print inputs
ld hl,0x2f0e			;screen position
in a,(0x02)			;read inputs port 2
call readinp			;print inputs
pop af				;restore a
ret     				;and return

readinp: ld b,0x08		;8 inputs to check
riloop: push bc
push hl
rrca    				;move current bit to carry
push af				;and save
ld a,0x28				;set a to "*"
jp c,over3				;and jump if button not pressed to print a "*"
ld a,0x25				;otherwise set a to " "
over3: call pchar		;print character
pop af				;restore registers
pop hl
inc hl
pop bc
dec b					;do it for all 8 inputs
jp nz,riloop			;continue if not complete
ret     				;and return

					;**PRINT STRING**
pstring: ld c,0x0d		;13 characters
					;**alternate entry point if not 13 characters**
psloop: ld a,(de)		;get character to print	
push de				;save de (character source)
push hl				;save hl (screen pos)
call pchar				;print character
pop hl				;retrieve hl
inc hl				;next position
pop de				;retrieve de
inc de				;next characted
dec c					;all printed?
jp  nz,psloop			;if not, loop back
ret     				;and return

					;**print single character a at screen position hl**
pchar: ld de,charset		;start of character set
push hl				;save hl
ld l,a				;get character
ld h,0x00
add hl,hl
add hl,hl
add hl,hl				;multiply by 8 (8 bytes per character)
add hl,de				;add to base
ex de,hl				;and move character set to de
pop hl				;retrieve hl
ld b,0x08				;8 bytes per character
pchloop: push bc			;store b
ld a,(de)				;get bits
ld (hl),a				;print onto screen
inc de				;next bit
ld bc,0x0020
add hl,bc				;next line
pop bc				;retrieve b
dec b
jp nz,pchloop			;if not 8 printed, continue loop
ret     				;and return


cls: ld hl,0x2400		;fill 2400-3FFF with 0 (clear screen RAM)
clsloop: ld (hl),0x00
inc hl
ld a,h
cp 0x40
jp nz,clsloop
ret     
					;Jump here if memory fault
memerr: ld b,0x01		;counter for error'd bit
memerrloop: rrca    
jp c,memex
inc b
jp memerrloop

memex: ld a,l			;select error code, l=low byte of erroring address
ld hl,charnum
rrca    
jp nc,memchosen			;if even address use 1-9
ld hl,charset			;if odd address use A-H
memchosen: ld a,b
rlca    
rlca    
rlca    
ld b,a
ld e,b
ld d,0x00
add hl,de				;calculate character data to use
ex de,hl
ld b,0x08				;8 lines
ld hl,0x3011			;screen position
prloop: ld a,(de)		;get character line
ld (hl),a				;print to screen
ld a,b				;save b
ld bc,0x0020			;next line
add hl,bc	
ld b,a				;restore b
inc de				;next character line
dec b
jp nz,prloop			;continue if not finished character

ld b,0x08				;8 lines
ld hl,0x3012			;screen position
ld de,charnum+104		;space character data
out (0x06),a			;watchdog
jp prloop				;and reprint loop

strings:
db 0x26,0x01,0x0c,0x0c,0x28,0x12,0x01,0x0d,0x13,0x28,0x0f,0x0b,0x28
db 0x26,0x03,0x08,0x05,0x03,0x0b,0x28,0x09,0x0e,0x10,0x15,0x14,0x13
db 0x26,0x03,0x08,0x05,0x03,0x0b,0x28,0x13,0x0f,0x15,0x0e,0x04,0x28

ports:
db 0x10,0x0f,0x12,0x14,0x1c,0x28,0x28,0x1b,0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22 ;PORT1  01234567

inputs:
db 0x25,0x15,0x06,0x0f,0x27,0x06
db 0x25,0x0d,0x09,0x13,0x13,0x0c
db 0x25,0x0c,0x01,0x15,0x27,0x08
db 0x25,0x09,0x0e,0x16,0x27,0x08
db 0x25,0x05,0x18,0x14,0x12,0x01

db 0x25,0x09,0x0e,0x16,0x27,0x1c
db 0x25,0x09,0x0e,0x16,0x27,0x1d
db 0x25,0x09,0x0e,0x16,0x27,0x1e
db 0x25,0x09,0x0e,0x16,0x27,0x1f
db 0x25,0x15,0x06,0x0f,0x27,0x08

charset:
db   0x00, 0x70, 0x88, 0xA8, 0xE8, 0x68, 0x08, 0xF0  ;"@" (0x00)
db   0x00, 0x20, 0x50, 0x88, 0x88, 0xF8, 0x88, 0x88  ;"A" (0x01)
db   0x00, 0x78, 0x88, 0x88, 0x78, 0x88, 0x88, 0x78  ;"B" (0x02)
db   0x00, 0x70, 0x88, 0x08, 0x08, 0x08, 0x88, 0x70  ;"C" (0x03)
db   0x00, 0x78, 0x88, 0x88, 0x88, 0x88, 0x88, 0x78  ;"D" (0x04)
db   0x00, 0xF8, 0x08, 0x08, 0x78, 0x08, 0x08, 0xF8  ;"E" (0x05)
db   0x00, 0xF8, 0x08, 0x08, 0x78, 0x08, 0x08, 0x08  ;"F" (0x06)
db   0x00, 0xF0, 0x08, 0x08, 0x08, 0xC8, 0x88, 0xF0  ;"G" (0x07)
db   0x00, 0x88, 0x88, 0x88, 0xF8, 0x88, 0x88, 0x88  ;"H" (0x08)
db   0x00, 0x70, 0x20, 0x20, 0x20, 0x20, 0x20, 0x70  ;"I" (0x09)
db   0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x88, 0x70  ;"J" (0x0a)
db   0x00, 0x88, 0x48, 0x28, 0x18, 0x28, 0x48, 0x88  ;"K" (0x0b)
db   0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0xF8  ;"L" (0x0c)
db   0x00, 0x88, 0xD8, 0xA8, 0xA8, 0x88, 0x88, 0x88  ;"M" (0x0d)
db   0x00, 0x88, 0x88, 0x98, 0xA8, 0xC8, 0x88, 0x88  ;"N" (0x0e)
db   0x00, 0x70, 0x88, 0x88, 0x88, 0x88, 0x88, 0x70  ;"O" (0x0f)
db   0x00, 0x78, 0x88, 0x88, 0x78, 0x08, 0x08, 0x08  ;"P" (0x10)
db   0x00, 0x70, 0x88, 0x88, 0x88, 0xA8, 0x48, 0xB0  ;"Q" (0x11)
db   0x00, 0x78, 0x88, 0x88, 0x78, 0x28, 0x48, 0x88  ;"R" (0x12)
db   0x00, 0x70, 0x88, 0x08, 0x70, 0x80, 0x88, 0x70  ;"S" (0x13)
db   0x00, 0xF8, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20  ;"T" (0x14)
db   0x00, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x70  ;"U" (0x15)
db   0x00, 0x88, 0x88, 0x88, 0x88, 0x88, 0x50, 0x20  ;"V" (0x16)
db   0x00, 0x88, 0x88, 0x88, 0xA8, 0xA8, 0xD8, 0x88  ;"W" (0x17)
db   0x00, 0x88, 0x88, 0x50, 0x20, 0x50, 0x88, 0x88  ;"X" (0x18)
db   0x00, 0x88, 0x88, 0x50, 0x20, 0x20, 0x20, 0x20  ;"Y" (0x19)
db   0x00, 0xF8, 0x80, 0x40, 0x20, 0x10, 0x08, 0xF8  ;"Z" (0x1a)
charnum:
db   0x00, 0x70, 0x88, 0xC8, 0xA8, 0x98, 0x88, 0x70  ;"0" (0x1b)
db   0x00, 0x20, 0x30, 0x20, 0x20, 0x20, 0x20, 0x70  ;"1" (0x1c)
db   0x00, 0x70, 0x88, 0x80, 0x60, 0x10, 0x08, 0xF8  ;"2" (0x1d)
db   0x00, 0xF8, 0x80, 0x40, 0x60, 0x80, 0x88, 0x70  ;"3" (0x1e)
db   0x00, 0x40, 0x60, 0x50, 0x48, 0xF8, 0x40, 0x40  ;"4" (0x1f)
db   0x00, 0xF8, 0x08, 0x78, 0x80, 0x80, 0x88, 0x70  ;"5" (0x20)
db   0x00, 0xE0, 0x10, 0x08, 0x78, 0x88, 0x88, 0x70  ;"6" (0x21)
db   0x00, 0xF8, 0x80, 0x40, 0x20, 0x10, 0x10, 0x10  ;"7" (0x22)
db   0x00, 0x70, 0x88, 0x88, 0x70, 0x88, 0x88, 0x70  ;"8" (0x23)
db   0x00, 0x70, 0x88, 0x88, 0xF0, 0x80, 0x40, 0x38  ;"9" (0x24)
db   0x00, 0x20, 0xA8, 0x70, 0x20, 0x70, 0xA8, 0x20  ;"*" (0x25)
db   0x00, 0x10, 0x20, 0x40, 0x80, 0x40, 0x20, 0x10  ;">" (0x26)
db   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20  ;"." (0x27)
db   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ;" " (0x28)
db   0x18, 0x3c, 0x7e, 0xdb, 0xff, 0x24, 0x5a, 0xa5  ;"A" (0x29)


